/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_LONGITUDINALINFO_H
#define DPC_IMPL_TYPE_LONGITUDINALINFO_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"
#include "impl_type_uint32.h"
#include "impl_type_uint8.h"
#include "impl_type_boolean.h"

namespace dpc {
struct LongitudinalInfo {
    ::Float desired_distance{ 0.0F };
    ::Float desired_speed{ 0.0F };
    ::Float set_speed{ 0.0F };
    ::Float set_distance{ 0.0F };
    ::Float yellow_border_distance{ 0.0F };
    ::Float red_border_distance{ 0.0F };
    ::UInt32 target_object_id{ 0U };
    ::Float sign_spd_limit_val_01{ 0.0F };
    ::Float sign_spd_limit_dist_01{ 0.0F };
    ::UInt8 sign_spd_limit_id_01{ 0U };
    ::Float sign_spd_limit_val_02{ 0.0F };
    ::Float sign_spd_limit_dist_02{ 0.0F };
    ::UInt8 sign_spd_limit_id_02{ 0U };
    ::Float curv_spd_limit_val{ 0.0F };
    ::Float curv_spd_limit_dist{ 0.0F };
    ::Boolean spd_limit_flg_last_cycle{ false };
    ::UInt8 spd_limit_type_last_cycle{ 0U };
    ::UInt8 spd_limit_id_last_cycle{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(desired_distance);
        fun(desired_speed);
        fun(set_speed);
        fun(set_distance);
        fun(yellow_border_distance);
        fun(red_border_distance);
        fun(target_object_id);
        fun(sign_spd_limit_val_01);
        fun(sign_spd_limit_dist_01);
        fun(sign_spd_limit_id_01);
        fun(sign_spd_limit_val_02);
        fun(sign_spd_limit_dist_02);
        fun(sign_spd_limit_id_02);
        fun(curv_spd_limit_val);
        fun(curv_spd_limit_dist);
        fun(spd_limit_flg_last_cycle);
        fun(spd_limit_type_last_cycle);
        fun(spd_limit_id_last_cycle);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(desired_distance);
        fun(desired_speed);
        fun(set_speed);
        fun(set_distance);
        fun(yellow_border_distance);
        fun(red_border_distance);
        fun(target_object_id);
        fun(sign_spd_limit_val_01);
        fun(sign_spd_limit_dist_01);
        fun(sign_spd_limit_id_01);
        fun(sign_spd_limit_val_02);
        fun(sign_spd_limit_dist_02);
        fun(sign_spd_limit_id_02);
        fun(curv_spd_limit_val);
        fun(curv_spd_limit_dist);
        fun(spd_limit_flg_last_cycle);
        fun(spd_limit_type_last_cycle);
        fun(spd_limit_id_last_cycle);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("desired_distance", desired_distance);
        fun("desired_speed", desired_speed);
        fun("set_speed", set_speed);
        fun("set_distance", set_distance);
        fun("yellow_border_distance", yellow_border_distance);
        fun("red_border_distance", red_border_distance);
        fun("target_object_id", target_object_id);
        fun("sign_spd_limit_val_01", sign_spd_limit_val_01);
        fun("sign_spd_limit_dist_01", sign_spd_limit_dist_01);
        fun("sign_spd_limit_id_01", sign_spd_limit_id_01);
        fun("sign_spd_limit_val_02", sign_spd_limit_val_02);
        fun("sign_spd_limit_dist_02", sign_spd_limit_dist_02);
        fun("sign_spd_limit_id_02", sign_spd_limit_id_02);
        fun("curv_spd_limit_val", curv_spd_limit_val);
        fun("curv_spd_limit_dist", curv_spd_limit_dist);
        fun("spd_limit_flg_last_cycle", spd_limit_flg_last_cycle);
        fun("spd_limit_type_last_cycle", spd_limit_type_last_cycle);
        fun("spd_limit_id_last_cycle", spd_limit_id_last_cycle);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("desired_distance", desired_distance);
        fun("desired_speed", desired_speed);
        fun("set_speed", set_speed);
        fun("set_distance", set_distance);
        fun("yellow_border_distance", yellow_border_distance);
        fun("red_border_distance", red_border_distance);
        fun("target_object_id", target_object_id);
        fun("sign_spd_limit_val_01", sign_spd_limit_val_01);
        fun("sign_spd_limit_dist_01", sign_spd_limit_dist_01);
        fun("sign_spd_limit_id_01", sign_spd_limit_id_01);
        fun("sign_spd_limit_val_02", sign_spd_limit_val_02);
        fun("sign_spd_limit_dist_02", sign_spd_limit_dist_02);
        fun("sign_spd_limit_id_02", sign_spd_limit_id_02);
        fun("curv_spd_limit_val", curv_spd_limit_val);
        fun("curv_spd_limit_dist", curv_spd_limit_dist);
        fun("spd_limit_flg_last_cycle", spd_limit_flg_last_cycle);
        fun("spd_limit_type_last_cycle", spd_limit_type_last_cycle);
        fun("spd_limit_id_last_cycle", spd_limit_id_last_cycle);
    }

    friend bool operator==(const ::dpc::LongitudinalInfo& lhs, const ::dpc::LongitudinalInfo& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.desired_distance - rhs.desired_distance)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.desired_speed - rhs.desired_speed)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.set_speed - rhs.set_speed)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.set_distance - rhs.set_distance)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.yellow_border_distance - rhs.yellow_border_distance)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.red_border_distance - rhs.red_border_distance)) < DBL_EPSILON) && (lhs.target_object_id == rhs.target_object_id) && (fabs(static_cast<double>(lhs.sign_spd_limit_val_01 - rhs.sign_spd_limit_val_01)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.sign_spd_limit_dist_01 - rhs.sign_spd_limit_dist_01)) < DBL_EPSILON) && (lhs.sign_spd_limit_id_01 == rhs.sign_spd_limit_id_01) && (fabs(static_cast<double>(lhs.sign_spd_limit_val_02 - rhs.sign_spd_limit_val_02)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.sign_spd_limit_dist_02 - rhs.sign_spd_limit_dist_02)) < DBL_EPSILON) && (lhs.sign_spd_limit_id_02 == rhs.sign_spd_limit_id_02) && (fabs(static_cast<double>(lhs.curv_spd_limit_val - rhs.curv_spd_limit_val)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.curv_spd_limit_dist - rhs.curv_spd_limit_dist)) < DBL_EPSILON) && (lhs.spd_limit_flg_last_cycle == rhs.spd_limit_flg_last_cycle) && (lhs.spd_limit_type_last_cycle == rhs.spd_limit_type_last_cycle) && (lhs.spd_limit_id_last_cycle == rhs.spd_limit_id_last_cycle);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_LONGITUDINALINFO_H
